"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @author Xuezi
 * @email xzj15859722542@hotmail.com
 * @create date 2018-06-22 11:49:57
 * @modify date 2018-06-22 11:49:57
 * @desc [description]
 */
const GRPC = require("grpc");
const protoLoader = require("@grpc/proto-loader");
class ClientContainer {
    static registryClient(target, path) {
        let packageDefinition = protoLoader.loadSync(path, {
            keepCase: true,
            longs: String,
            enums: String,
            defaults: true,
            oneofs: true
        });
        let protoDescriptor = GRPC.loadPackageDefinition(packageDefinition);
        const packages = Object.keys(protoDescriptor);
        for (let packageKey of packages) {
            for (let key in protoDescriptor[packageKey]) {
                if (protoDescriptor[packageKey][key].hasOwnProperty('service')) {
                    this.clients.push({ target, service: protoDescriptor[packageKey][key] });
                }
            }
        }
    }
    static registryRoute(target, property, key) {
        this.routes.push({ target, property, key });
    }
    static generateRouteFunc(route) {
        let func = this._generateRouteFunc(route.property.value, route.target, route.key);
        Object.defineProperty(route.target.constructor.prototype, route.key, {
            enumerable: false,
            configurable: false,
            writable: false,
            value: func
        });
        // route.target.constructor.prototype[route.key] = func;
        route.property.value = func;
    }
    static _generateRouteFunc(route, target, key) {
        let clientContainer = this
            .getClient(target.constructor);
        let _func = clientContainer.client[key];
        let { requestStream, responseStream } = _func;
        let func;
        if (requestStream && responseStream) {
            func = function () {
                return __awaiter(this, arguments, void 0, function* () {
                    if (_func instanceof Function) {
                        let _routeFunc = route;
                        let _writeData = arguments[0] || {};
                        let call = _func.call(clientContainer.client);
                        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                            let result;
                            let error;
                            call.on('data', (chunk) => __awaiter(this, void 0, void 0, function* () {
                                result = yield _routeFunc(_writeData, chunk);
                                if (!result) {
                                    result = chunk;
                                }
                                call.end();
                            }));
                            call.on('error', e => {
                                error = e;
                                call.end();
                            });
                            call.on('end', () => {
                                if (error) {
                                    return reject(error);
                                }
                                resolve(result);
                            });
                            call.write(_writeData);
                        }));
                    }
                });
            };
        }
        else if (requestStream && !responseStream) {
            func = function () {
                return __awaiter(this, arguments, void 0, function* () {
                    let _routeFunc = route;
                    let _writeData = arguments[0] || {};
                    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                        let call = _func.call(clientContainer.client, (err, res) => __awaiter(this, void 0, void 0, function* () {
                            if (err)
                                return reject(err);
                            let result = yield _routeFunc(_writeData, res);
                            if (!result) {
                                result = res;
                            }
                            resolve(result);
                        }));
                        call.on('error', e => {
                            reject(e);
                            call.end && call.end();
                        });
                        call.write(_writeData);
                        call.end();
                    }));
                });
            };
        }
        else if (!requestStream && responseStream) {
            func = function () {
                return __awaiter(this, arguments, void 0, function* () {
                    let _routeFunc = route;
                    let _writeData = arguments[0] || {};
                    return new Promise((resolve, reject) => {
                        let call = _func.call(clientContainer.client, _writeData);
                        call.on('error', e => {
                            reject(e);
                            call.end && call.end();
                        });
                        call.on('end', () => {
                            call.end && call.end();
                        });
                        call.on('data', (chunk) => __awaiter(this, void 0, void 0, function* () {
                            let result = yield _routeFunc(_writeData, chunk);
                            if (!result) {
                                result = chunk;
                            }
                            resolve(result);
                        }));
                    });
                });
            };
        }
        else if (!requestStream && !responseStream) {
            func = function () {
                return __awaiter(this, arguments, void 0, function* () {
                    let _routeFunc = route;
                    let _writeData = arguments[0] || {};
                    return new Promise((resolve, reject) => {
                        _func.call(clientContainer.client, _writeData, (err, res) => __awaiter(this, void 0, void 0, function* () {
                            if (err)
                                return reject(err);
                            let result = yield _routeFunc(_writeData, res);
                            if (!result) {
                                result = res;
                            }
                            resolve(result);
                        }));
                    });
                });
            };
        }
        return func;
    }
    static getRoutes(target) {
        let routes = this.routes.filter(route => route.target.constructor === target);
        return routes;
    }
    static getRoute(target, key) {
        let route = this.routes.find(route => (route.key === key && route.target === target));
        return route;
    }
    static getClient(target) {
        let client = this.clients.find(client => client.target === target);
        return client;
    }
}
ClientContainer.clients = [];
ClientContainer.routes = [];
exports.ClientContainer = ClientContainer;
//# sourceMappingURL=ClientContainer.js.map