"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @author Xuezi
 * @email xzj15859722542@hotmail.com
 * @create date 2018-06-22 11:50:05
 * @modify date 2018-06-22 11:50:05
 * @desc [description]
 */
const GRPC = require("grpc");
const FS = require("fs");
const SettingRegistry_1 = require("./SettingRegistry");
const ClientContainer_1 = require("./ClientContainer");
class RpcClientRegistry {
    static get ca() {
        return this._ca;
    }
    static get client_cert() {
        return this._client_cert;
    }
    static get client_key() {
        return this._client_key;
    }
    static get port() {
        return this._port;
    }
    static get host() {
        return this._host;
    }
    static get credentials() {
        return this._credentials;
    }
    static start() {
        this._loadSettings();
        this._addAuth();
        this._registryClient();
    }
    static _registryClient() {
        for (let clientContainer of ClientContainer_1.ClientContainer.clients) {
            let client = new clientContainer.service(`${this.host}:${this.port}`, this.credentials);
            clientContainer.client = client;
            this._registryRoute(clientContainer);
        }
    }
    static _registryRoute(clientContainer) {
        let target = clientContainer.target;
        let routes = ClientContainer_1.ClientContainer.getRoutes(target);
        for (let route of routes) {
            ClientContainer_1.ClientContainer.generateRouteFunc(route);
        }
    }
    static _loadSettings() {
        let { host, port, ca, client_key, client_cert } = SettingRegistry_1.SettingRegistry.settings;
        this._host = host;
        this._port = port;
        this._ca = ca;
        this._client_key = client_key;
        this._client_cert = client_cert;
    }
    static _addAuth() {
        if (!this.ca || !this.client_cert || !this.client_key) {
            this._credentials = GRPC.credentials.createInsecure();
        }
        else {
            let ca = FS.readFileSync(this.ca);
            let cert = FS.readFileSync(this.client_cert);
            let key = FS.readFileSync(this.client_key);
            this._credentials = GRPC.credentials.createSsl(ca, key, cert);
        }
    }
}
exports.RpcClientRegistry = RpcClientRegistry;
//# sourceMappingURL=RpcClientRegistry.js.map